// PBE.java
// Public Based Encryption (Password)

import java.security.*;
import java.security.spec.*;
import javax.crypto.*;
import javax.crypto.spec.*;
import java.util.*;
import java.io.*;

public class PBE {

	private static final int ITERATIONS = 1000;
	
	public static String encrypt(char[] password, String plaintext) {
	
		byte[ ] salt = new byte[8];
		Random random = new Random();
		random.nextBytes(salt);
		
		PBEKeySpec keySpec = new PBEKeySpec(password);
		
		try {
			SecretKeyFactory keyFactory = SecretKeyFactory.getInstance("PBEWithMD5AndDES");
			SecretKey key = keyFactory.generateSecret(keySpec);
			
			PBEParameterSpec paramSpec = new PBEParameterSpec(salt, ITERATIONS);
			Cipher cipher = Cipher.getInstance("PBEWithMD5AndDES");
			cipher.init(Cipher.ENCRYPT_MODE, key, paramSpec);
		
			byte[ ] ciphertext = cipher.doFinal(plaintext.getBytes("UTF8"));
			
			String saltString = new String(salt);
			String ciphertextString = new String(ciphertext);
			
			return saltString+ciphertextString;
		}
		
		catch (NoSuchAlgorithmException e1) {		
			System.out.println("Algoritmo non supportato");	
		}
		
		catch (NoSuchPaddingException e2) {		
			System.out.println("Padding non supportato");	
		}
		
		catch (InvalidKeySpecException e3) {
			System.out.println("Specificazione della chiave non valida");	
		}
		
		catch (InvalidKeyException e4) {
			System.out.println("Chiave non valida");
		}
		
		catch (InvalidAlgorithmParameterException e5) {
			System.out.println("Parametro non valido");
		}
		
		catch (IllegalBlockSizeException e6) {
			System.out.println("Dimensione blocco illegale");
		}
		
		catch (BadPaddingException e7) {
			System.out.println("Bad Padding");
		}
		
		catch (UnsupportedEncodingException e8) {
			System.out.println("Codifica non supportata");
		}
		
		return "Error";
		
	}
	
	public static String decrypt(char[] password, String input) {
				
		try {
			
			String saltString = input.substring(0,8);
			String ciphertextString = input.substring(8,input.length());
			
			byte[ ] salt = saltString.getBytes("UTF8");
			byte[ ] ciphertext = ciphertextString.getBytes("UTF8");
			
			PBEKeySpec keySpec = new PBEKeySpec(password);
			
			SecretKeyFactory keyFactory =SecretKeyFactory.getInstance("PBEWithMD5AndDES");
			SecretKey key = keyFactory.generateSecret(keySpec);
				
			PBEParameterSpec paramSpec = new PBEParameterSpec(salt,ITERATIONS);
			Cipher cipher = Cipher.getInstance("PBEWithMD5AndDES");
			cipher.init(Cipher.DECRYPT_MODE, key, paramSpec);
				
			byte[ ] plaintextArray = cipher.doFinal(ciphertext);
				
			return new String(plaintextArray);
		}
		
		catch (NoSuchAlgorithmException e1) {		
			System.out.println("Algoritmo non supportato");
		}
		
		catch (NoSuchPaddingException e2) {		
			System.out.println("Padding non supportato");	
		}
		
		catch (InvalidKeySpecException e3) {
			System.out.println("Specificazione della chiave non valida");
		}
		
		catch (InvalidKeyException e4) {
			System.out.println("Chiave non valida");
		}
		
		catch (InvalidAlgorithmParameterException e5) {
			System.out.println("Parametro non valido");
		}
		
		catch (IllegalBlockSizeException e6) {
			System.out.println("Dimensione blocco illegale");
		}
		
		catch (BadPaddingException e7) {
			System.out.println("Bad Padding");
		}	
				
		catch (UnsupportedEncodingException e8) {
			System.out.println("Codifica non supportata");
		}
		
		return "Error";
		
	}
	
	public static void main(String[] args) {
		System.out.println("PASSWORD-BASED ENCRYPTION");
		char[] password;
		String text;
		if(args.length!=2) {
			String pwd="password";	
			password=new char[pwd.length()];
			pwd.getChars(0,pwd.length(),password,0);
			text="Hello World!";
		}
		else {
			password=new char[args[0].length()];
			args[0].getChars(0,args[0].length(),password,0);
			text=args[1];
		}
		System.out.println("\nTEXT:\n"+text);
		String cipherText=encrypt(password,text);
		System.out.println("\nCIPHERTEXT:\n"+cipherText);
		String decryptedText=decrypt(password,cipherText);
		System.out.println("\nDECRYPTED TEXT:\n"+decryptedText);
	} 

}

