// PBE2.java
/* 
   Public Based Encryption (Password)
   Ridefinizione dei metodi per la cifratura di una chiave simmetrica
   generata con TripleDES mediante WRAP_MODE e UNWRAP_MODE
*/

import java.security.*;
import java.security.spec.*;
import javax.crypto.*;
import javax.crypto.spec.*;
import java.util.*;

public class PBE2 {

	private static final int ITERATIONS = 1000;
	
	public static byte[] encrypt(char[] password, Key keyPassword) {
	
		byte[] salt = new byte[8];
		Random random = new Random();
		random.nextBytes(salt);
		
		PBEKeySpec keySpec = new PBEKeySpec(password);
		
		try {
			SecretKeyFactory keyFactory = SecretKeyFactory.getInstance("PBEWithMD5AndDES");
			SecretKey key = keyFactory.generateSecret(keySpec);
			
			PBEParameterSpec paramSpec = new PBEParameterSpec(salt, ITERATIONS);
			Cipher cipher = Cipher.getInstance("PBEWithMD5AndDES");
			
			// senza incapsulamento
			cipher.init(Cipher.WRAP_MODE, key, paramSpec);
			byte[] keyEncrypted = cipher.wrap(keyPassword);
			
			int length=keyEncrypted.length+8;
			byte result[]=new byte[length];
			for(int i=0; i<8; i++) result[i]=salt[i];
			for(int i=8; i<length; i++) result[i]=keyEncrypted[i-8];
			
			return result;
		}
		
		catch (NoSuchAlgorithmException e1) {		
			System.out.println("Algoritmo non supportato");	
			
		}
		
		catch (NoSuchPaddingException e2) {		
			System.out.println("Padding non supportato");		
		}
		
		catch (InvalidKeySpecException e3) {
			System.out.println("Specificazione della chiave non valida");
		}
		
		catch (InvalidKeyException e4) {
			System.out.println("Chiave non valida");	
		}
		
		catch (InvalidAlgorithmParameterException e5) {
			System.out.println("Parametro non valido");
		}
		
		catch (IllegalBlockSizeException e6) {
			System.out.println("Dimensione blocco illegale");
		}
		
		return salt;
		
	}
	
	public static Key decrypt(char[] password,  byte[] input) {
			
		Key secretKey = null;
		
		byte salt[] = new byte[8];
		byte keyEncrypted[] = new byte[input.length-8];
		
		for(int i=0; i<8; i++) salt[i]=input[i];
		for(int i=8; i<input.length; i++) keyEncrypted[i-8]=input[i];
			
		PBEKeySpec keySpec = new PBEKeySpec(password);
		
		try {
			SecretKeyFactory keyFactory =SecretKeyFactory.getInstance("PBEWithMD5AndDES");
			SecretKey key = keyFactory.generateSecret(keySpec);
				
			PBEParameterSpec paramSpec = new PBEParameterSpec(salt,ITERATIONS);
			Cipher cipher = Cipher.getInstance("PBEWithMD5AndDES");
			
			// senza incapsulamento
			cipher.init(Cipher.UNWRAP_MODE, key, paramSpec);
			secretKey = cipher.unwrap(keyEncrypted,"TripleDES",Cipher.SECRET_KEY);
						
			
		}
		
		catch (NoSuchAlgorithmException e1) {		
			System.out.println("Algoritmo non supportato");
		}
		
		catch (NoSuchPaddingException e2) {		
			System.out.println("Padding non supportato");	
		}
		
		catch (InvalidKeySpecException e3) {
			System.out.println("Specificazione della chiave non valida");	
		}
		
		catch (InvalidKeyException e4) {
			System.out.println("Chiave non valida");
		}
		
		catch (InvalidAlgorithmParameterException e5) {
			System.out.println("Parametro non valido");	
		}
			
		return secretKey;
	}

}